@@ -11,12 +11,11 @@
 #    under the License.
 
 import json
+import queue
 import threading
 import uuid
 
 import requests
-import six
-from six.moves import queue
 
 from etcd3gw import exceptions
 from etcd3gw.lease import Lease
@@ -41,8 +40,9 @@ _EXCEPTIONS_BY_CODE = {
 
 class Etcd3Client(object):
     def __init__(self, host='localhost', port=2379, protocol="http",
-                 ca_cert=None, cert_key=None, cert_cert=None, timeout=None):
-        """Construct an client to talk to etcd3's grpc-gateway's /v3alpha HTTP API
+                 ca_cert=None, cert_key=None, cert_cert=None, timeout=None,
+                 api_path='/v3alpha/'):
+        """Construct an client to talk to etcd3's grpc-gateway's /v3 HTTP API
 
         :param host:
         :param port:
@@ -59,9 +59,10 @@ class Etcd3Client(object):
             self.session.verify = ca_cert
         if cert_cert is not None and cert_key is not None:
             self.session.cert = (cert_cert, cert_key)
+        self.api_path = api_path
 
     def get_url(self, path):
-        """Construct a full url to the v3alpha API given a specific path
+        """Construct a full url to the v3 API given a specific path
 
         :param path:
         :return: url
@@ -69,7 +70,7 @@ class Etcd3Client(object):
         host = ('[' + self.host + ']' if (self.host.find(':') != -1)
                 else self.host)
         base_url = self.protocol + '://' + host + ':' + str(self.port)
-        return base_url + '/v3alpha/' + path.lstrip("/")
+        return base_url + self.api_path + path.lstrip("/")
 
     def post(self, *args, **kwargs):
         """helper method for HTTP POST
@@ -81,13 +82,16 @@ class Etcd3Client(object):
         try:
             resp = self.session.post(*args, **kwargs)
             if resp.status_code in _EXCEPTIONS_BY_CODE:
-                raise _EXCEPTIONS_BY_CODE[resp.status_code](resp.reason)
+                raise _EXCEPTIONS_BY_CODE[resp.status_code](
+                    resp.text,
+                    resp.reason
+                )
             if resp.status_code != requests.codes['ok']:
                 raise exceptions.Etcd3Exception(resp.text, resp.reason)
         except requests.exceptions.Timeout as ex:
-            raise exceptions.ConnectionTimeoutError(six.text_type(ex))
+            raise exceptions.ConnectionTimeoutError(str(ex))
         except requests.exceptions.ConnectionError as ex:
-            raise exceptions.ConnectionFailedError(six.text_type(ex))
+            raise exceptions.ConnectionFailedError(str(ex))
         return resp.json()
 
     def status(self):
@@ -128,7 +132,7 @@ class Etcd3Client(object):
             id = str(uuid.uuid4())
         return Lock(id, ttl=ttl, client=self)
 
-    def create(self, key, value):
+    def create(self, key, value, lease=None):
         """Atomically create the given key only if the key doesn't exist.
 
         This verifies that the create_revision of a key equales to 0, then
@@ -138,6 +142,7 @@ class Etcd3Client(object):
         :param key: key in etcd to create
         :param value: value of the key
         :type value: bytes or string
+        :param lease: lease to connect with, optional
         :returns: status of transaction, ``True`` if the create was
                   successful, ``False`` otherwise
         :rtype: bool
@@ -159,6 +164,8 @@ class Etcd3Client(object):
             }],
             'failure': []
         }
+        if lease:
+            txn['success'][0]['request_put']['lease'] = lease.id
         result = self.transaction(txn)
         if 'succeeded' in result:
             return result['succeeded']
@@ -224,12 +231,11 @@ class Etcd3Client(object):
         if metadata:
             def value_with_metadata(item):
                 item['key'] = _decode(item['key'])
-                value = _decode(item.pop('value'))
+                value = _decode(item.pop('value', ''))
                 return value, item
-
             return [value_with_metadata(item) for item in result['kvs']]
-        else:
-            return [_decode(item['value']) for item in result['kvs']]
+
+        return [_decode(item.get('value', '')) for item in result['kvs']]
 
     def get_all(self, sort_order=None, sort_target='key'):
         """Get all keys currently stored in etcd.
